############################################################
#   Phylogenetic measures on a set of bayesian trees       #
############################################################

library(picante)
library(ape)
library(caper)
library(plyr) 
library(phylocomr)
library(geiger)
library(colortools)
library(ggplot2)
library(ggthemes)
library(viridis)
library(forcats)

#1. Read MrBayes outputs
tree<-read.nexus("../CIPRES\ folder/infile.nex.con.tre")

#2. Read MrBayes outputs

run1 <- read.table("../CIPRES\ folder/infile.nex.run1.p",header=T,skip=1)
run2 <- read.table("../CIPRES\ folder/infile.nex.run2.p",header=T,skip=1)

rangerun1 <- c(-21116.76, -21065.22) # data from Tracer 
HPDrun1 <- subset(run1,LnL>rangerun1[1] & LnL<rangerun1[2])

rangerun2 <- c(-21117.83, -21066.54) # data from Tracer
HPDrun2 <- subset(run2,LnL>rangerun2[1] & LnL<rangerun2[2])

trees1 <- read.nexus("../CIPRES\ folder/infile.nex.run1.t")
trees2 <- read.nexus("../CIPRES\ folder/infile.nex.run2.t")

#2a. ## select trees in 95% HPD
genrun1 <- paste("gen.",HPDrun1$Gen,sep="")
x <- which(names(trees1) %in% genrun1)
treesHPD1 <- trees1[x] 

genrun2 <- paste("gen.",HPDrun2$Gen,sep="")
y <- which(names(trees2) %in% genrun2)
treesHPD2 <- trees2[y]

#3. ## combine trees from run1 and run2
bayesT <- c(treesHPD1,treesHPD2)  

## root tree
trees <- lapply(bayesT, root,"Arundina_graminifolia", node = NULL, resolve.root = T)

#class(trees) <- "multiPhylo"

#4. load database
Coelogyninae_df <- read.csv("../Coelogyninae_DB_RK.csv", sep=",", dec=".", header = TRUE)

save.image("trees_and_database.RData")

#load("trees_and_database.RData")

#4. choose subsample in 95% HPD
subsetsize <- 1000
posteriorsample <- sample(trees, size=subsetsize)
class(posteriorsample)
write.nexus(posteriorsample, file = "posteriorsample.nex")


####################################
#9. #  level 2 states              #
####################################

medi <- Coelogyninae_df[which(Coelogyninae_df$Use_Level1state == "Medicines"),c(4,32)]
medtab <- table(medi)

##replace names in database with treetip names
#rownames(medtab)[which(rownames(medtab) %in% transtab[,2])] <- transtab[which(transtab[,2] %in% rownames(medtab)),1] 
#medtab <- as.matrix(as.data.frame.matrix(medtab)) # species names in rows

m <- matrix(0,length(trees[[1]]$tip.label[-which(trees[[1]]$tip.label %in% rownames(medtab))]),ncol(medtab))
rownames(m) <- trees[[1]]$tip.label[-which(trees[[1]]$tip.label %in% rownames(medtab))]
phylo_lev2 <- rbind(medtab,m)

lev2_phylo <- as.data.frame(phylo_lev2) # species names in rows
lev2_phylo[lev2_phylo != 0] <- 1 # replace all non-zero values with 1 (binary data)
lev2_phylo$Species <- rownames(lev2_phylo)
colnames(lev2_phylo)
############################################ begin function
phylo.d.lev2 <- function(trees, subset, nper){
  
  if (subset == 0){
    subtrees <- trees
  }
  
  if (subset != 0){
    subtrees <- sample(trees, size=subset)
  }
  
  subtrees <- lapply(subtrees, di2multi, tol = 1e-06)  
  class(subtrees) <- "multiPhylo"
  
  level2 <- lapply(subtrees,comparative.data,lev2_phylo, Species, force.root=T) 
  
#  for (i in 1:length(level2)){
#    names(level2[[i]]$data) <- gsub("\\s","", names(level2[[i]]$data)) #remove spaces to create #valid column names
#    names(level2[[i]]$data) <- gsub( "[[:punct:]]" , "" , names(level2[[i]]$data))    
#  }
  
  PhyloD_lev2 <- matrix(0,(length(names(level2[[1]]$data))*length(subtrees)),7)
  colnames(PhyloD_lev2) <- c("Binary variable", "0's","1's","N.perm","D","Prob.random","Prob.Brownian")
  
  var <- c(8,5,5,10,1,2,3)
  var2 <- c(1,1,2,1,1,1,1)
  
  # Start Lev2
  for(j in 1:length(level2)){
    print(c("Abnormalities",j))
    for (i in 1:7){
      PhyloD_lev2[j,i] <- phylo.d(level2[[j]], binvar = Abnormalities, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Circulatory_System_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+length(level2),i] <- phylo.d(level2[[j]], binvar = Circulatory_System_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Digestive_System_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+(2*length(level2)),i] <- phylo.d(level2[[j]], binvar = Digestive_System_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Genitourinary_System_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+(3*length(level2)),i] <- phylo.d(level2[[j]], binvar = Genitourinary_System_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("III_Defined_Symptoms",j))
    for (i in 1:7){
      PhyloD_lev2[j+(4*length(level2)),i] <- phylo.d(level2[[j]], binvar = III_Defined_Symptoms, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Infections_Infestations",j))
    for (i in 1:7){
      PhyloD_lev2[j+(5*length(level2)),i] <- phylo.d(level2[[j]], binvar = Infections_Infestations, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Inflammation",j))
    for (i in 1:7){
      PhyloD_lev2[j+(6*length(level2)),i] <- phylo.d(level2[[j]], binvar = Inflammation, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Injuries",j))
    for (i in 1:7){
      PhyloD_lev2[j+(7*length(level2)),i] <- phylo.d(level2[[j]], binvar = Injuries, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Mental_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+(8*length(level2)),i] <- phylo.d(level2[[j]], binvar = Mental_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Metabolic_System_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+(9*length(level2)),i] <- phylo.d(level2[[j]], binvar = Metabolic_System_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Muscular_Skeletal_System_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+(10*length(level2)),i] <- phylo.d(level2[[j]], binvar = Muscular_Skeletal_System_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Nervous_System_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+(11*length(level2)),i] <- phylo.d(level2[[j]], binvar = Nervous_System_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Nutritional_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+(12*length(level2)),i] <- phylo.d(level2[[j]], binvar = Nutritional_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Pain",j))
    for (i in 1:7){
      PhyloD_lev2[j+(13*length(level2)),i] <- phylo.d(level2[[j]], binvar = Pain, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Poisonings",j))
    for (i in 1:7){
      PhyloD_lev2[j+(14*length(level2)),i] <- phylo.d(level2[[j]], binvar = Poisonings, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Respiratory_System_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+(15*length(level2)),i] <- phylo.d(level2[[j]], binvar = Respiratory_System_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Sensory_System_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+(16*length(level2)),i] <- phylo.d(level2[[j]], binvar = Sensory_System_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("Skin_Subcutaneous_Cellular_Tissue_Disorders",j))
    for (i in 1:7){
      PhyloD_lev2[j+(17*length(level2)),i] <- phylo.d(level2[[j]], binvar = Skin_Subcutaneous_Cellular_Tissue_Disorders, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(level2)){
    print(c("unknown",j))
    for (i in 1:7){
      PhyloD_lev2[j+(18*length(level2)),i] <- phylo.d(level2[[j]], binvar = unknown, permut= nper)[[var[i]]][var2[i]]
    }
  }

  star1 <- as.numeric(PhyloD_lev2[,6]) <= 0.05 # * p â‰¤ 0.05
  star2 <- as.numeric(PhyloD_lev2[,6]) <= 0.01 # * p â‰¤ 0.01
  star3 <- as.numeric(PhyloD_lev2[,6]) <= 0.001 # * p â‰¤ 0.001
  star4 <- as.numeric(PhyloD_lev2[,6]) <= 0.0001 # * p â‰¤ 0.0001
  star_Brown <- as.numeric(PhyloD_lev2[,7]) >= 0.05 # p â‰¥ 0.05 (Brownian)
  PhyloD_lev2 <- cbind(PhyloD_lev2, star1, star2, star3, star4,star_Brown)
  
  median_D <- t(sapply(by(as.numeric(PhyloD_lev2[,5]),PhyloD_lev2[,1],median),identity))
  range_max <- t(sapply(by(as.numeric(PhyloD_lev2[,5]),PhyloD_lev2[,1],max),identity))
  range_min <- t(sapply(by(as.numeric(PhyloD_lev2[,5]),PhyloD_lev2[,1],min),identity))
  sig_code1 <- t(sapply(by(as.logical(PhyloD_lev2[,8]),PhyloD_lev2[,1],mean),identity)) # e.g. sig_code1 value of 0.95 means that 95% of the p-values were â‰¤ 0.05
  sig_code2 <- t(sapply(by(as.logical(PhyloD_lev2[,9]),PhyloD_lev2[,1],mean),identity)) # e.g. sig_code2 value of 0.95 means that 95% of the p-values were â‰¤ 0.01
  sig_code3 <- t(sapply(by(as.logical(PhyloD_lev2[,10]),PhyloD_lev2[,1],mean),identity))
  sig_code4 <- t(sapply(by(as.logical(PhyloD_lev2[,11]),PhyloD_lev2[,1],mean),identity))
  sig_Brown <- t(sapply(by(as.logical(PhyloD_lev2[,12]),PhyloD_lev2[,1],mean),identity)) # e.g. sig_Brown value of 0.95 means that 95% of the p-values were not distinguishable from Brownian motion (pâ‰¥0.05)
  
  Sum_stats <- rbind(median_D, range_min, range_max, sig_code1, sig_code2, sig_code3, sig_code4, sig_Brown)
  rownames(Sum_stats) <- c("median_D","min_D","max_D","sig_code1", "sig_code2","sig_code3","sig_code4","sig_Brown")
  
  output <- list()
  output[[1]] <- PhyloD_lev2
  output[[2]] <- Sum_stats
  
  return(output)
}
############################################################ end function 

phyloD_lev2 <- phylo.d.lev2(posteriorsample,0,1000)
write.table(phyloD_lev2[[1]],"phyloD_lev2a.txt")
write.table(phyloD_lev2[[2]],"phyloD_lev2b.txt")

write.csv((phyloD_lev2[[1]]),"phyloD_lev2a.csv")
write.csv(t(phyloD_lev2[[2]]),"phyloD_lev2b.csv")

save.image("level2_data.RData")

####################################
#10. Antimicrobial response         #
####################################

anti <- Coelogyninae_df[which(Coelogyninae_df$antimicrobial.response != "\\"),c(5,52)]
antitab <- table(anti)

## replace names in database with treetip names
#rownames(antitab)[which(rownames(antitab) %in% transtab[,2])] <- transtab[which(transtab[,2] %in% rownames(antitab)),1] 
antitab <- as.matrix(as.data.frame.matrix(antitab)) # species names in rows

m <- matrix(0,length(trees[[1]]$tip.label[-which(trees[[1]]$tip.label %in% rownames(antitab))]),ncol(antitab))
rownames(m) <- trees[[1]]$tip.label[-which(trees[[1]]$tip.label %in% rownames(antitab))]
phylo_anti <- rbind(antitab,m)
phylo_anti[which(phylo_anti[,2]!=0),c(1,3)] <- 0 
phylo_anti[which(phylo_anti[,1]!=0),3] <- 0 

anti_phylo <- as.data.frame(phylo_anti) # species names in rows
anti_phylo[anti_phylo != 0] <- 1 # replace all non-zero values with 1 (binary data)
anti_phylo$Species <- rownames(anti_phylo)
colnames(anti_phylo)

############################################ begin function
phylo.d.anti <- function(trees, subset, nper){
  
  if (subset == 0){
    subtrees <- trees
  }
  
  if (subset != 0){
    subtrees <- sample(trees, size=subset)
  }
  
  subtrees <- lapply(subtrees, di2multi, tol = 1e-06) 
  class(subtrees) <- "multiPhylo"
  
  antiC <- lapply(subtrees,comparative.data,anti_phylo, Species, force.root=T) 
  
#  for (i in 1:length(antiC)){
#    names(antiC[[i]]$data) <- gsub("\\s","", names(antiC[[i]]$data)) #remove spaces to create valid #column names
#    names(antiC[[i]]$data) <- gsub( "[[:punct:]]" , "" , names(antiC[[i]]$data))    
#  }
  
  PhyloD_anti <- matrix(0,(length(names(antiC[[1]]$data))*length(subtrees)),7)
  colnames(PhyloD_anti) <- c("Binary variable", "0's","1's","N.perm","D","Prob.random","Prob.Brownian")
  
  var <- c(8,5,5,10,1,2,3)
  var2 <- c(1,1,2,1,1,1,1)
  
  # Start Lev2
  for(j in 1:length(antiC)){
    print(c("no",j))
    for (i in 1:7){
      PhyloD_anti[j,i] <- phylo.d(antiC[[j]], binvar = no, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(antiC)){
    print(c("possible",j))
    for (i in 1:7){
      PhyloD_anti[j+length(antiC),i] <- phylo.d(antiC[[j]], binvar = possible, permut= nper)[[var[i]]][var2[i]]
    }
  }
  for(j in 1:length(antiC)){
    print(c("unknown_process",j))
    for (i in 1:7){
      PhyloD_anti[j+(2*length(antiC)),i] <- phylo.d(antiC[[j]], binvar = unknown_process, permut= nper)[[var[i]]][var2[i]]
    }
  }
  
  star1 <- as.numeric(PhyloD_anti[,6]) <= 0.05 # * p ≤ 0.05
  star2 <- as.numeric(PhyloD_anti[,6]) <= 0.01 # * p ≤ 0.01
  star3 <- as.numeric(PhyloD_anti[,6]) <= 0.001 # * p ≤ 0.001
  star4 <- as.numeric(PhyloD_anti[,6]) <= 0.0001 # * p ≤ 0.0001
  star_Brown <- as.numeric(PhyloD_anti[,7]) >= 0.05 # p ≥ 0.05 (Brownian)
  PhyloD_anti <- cbind(PhyloD_anti, star1, star2, star3, star4,star_Brown)
  
  median_D <- t(sapply(by(as.numeric(PhyloD_anti[,5]),PhyloD_anti[,1],median),identity))
  range_max <- t(sapply(by(as.numeric(PhyloD_anti[,5]),PhyloD_anti[,1],max),identity))
  range_min <- t(sapply(by(as.numeric(PhyloD_anti[,5]),PhyloD_anti[,1],min),identity))
  sig_code1 <- t(sapply(by(as.logical(PhyloD_anti[,8]),PhyloD_anti[,1],mean),identity)) # e.g. sig_code1 value of 0.95 means that 95% of the p-values were ≤ 0.05
  sig_code2 <- t(sapply(by(as.logical(PhyloD_anti[,9]),PhyloD_anti[,1],mean),identity)) # e.g. sig_code2 value of 0.95 means that 95% of the p-values were ≤ 0.01
  sig_code3 <- t(sapply(by(as.logical(PhyloD_anti[,10]),PhyloD_anti[,1],mean),identity))
  sig_code4 <- t(sapply(by(as.logical(PhyloD_anti[,11]),PhyloD_anti[,1],mean),identity))
  sig_Brown <- t(sapply(by(as.logical(PhyloD_anti[,12]),PhyloD_anti[,1],mean),identity)) # e.g. sig_Brown value of 0.95 means that 95% of the p-values were not distinguishable from Brownian motion (p≥0.05)
  
  Sum_stats <- rbind(median_D, range_min, range_max, sig_code1, sig_code2, sig_code3, sig_code4, sig_Brown)
  rownames(Sum_stats) <- c("median_D","min_D","max_D","sig_code1", "sig_code2","sig_code3","sig_code4","sig_Brown")
  
  output <- list()
  output[[1]] <- PhyloD_anti
  output[[2]] <- Sum_stats
  
  return(output)
}
############################################################ end function 

phyloD_anti <- phylo.d.anti(posteriorsample,0,1000)
write.table(phyloD_anti[[1]],"phyloD_antia.txt")
write.table(phyloD_anti[[2]],"phyloD_antib.txt")

write.csv(t(phyloD_anti[[1]]),"phyloD_antia.csv")
write.csv(t(phyloD_anti[[2]]),"phyloD_antib.csv")

as.numeric.factor <- function(x) {as.numeric(levels(x))[x]}

table1<-phyloD_anti[[1]]
table2<-phyloD_anti[[2]]

table1<-as.data.frame(table1)
table1$D<-as.numeric.factor(table1$D)

save.image("antimicrobial_response_data.RData")


####################################
#         MNTD (with comdistnt)    #
####################################

medtab2 <- phylo_lev2#[which(rownames(phylo_lev2) %in% trees[[1]]$tip.label),]
medtab2 <- as.data.frame(medtab2)
medtab2 <- medtab2
medtab2$Species <- rownames(medtab2)

antitab2 <- phylo_anti#[which(rownames(phylo_anti) %in% trees[[1]]$tip.label),]
antitab2 <- as.data.frame(antitab2)
antitab2 <-antitab2
antitab2$Species <- rownames(antitab2)

#Reorder columns so "possible" appears first. This because we will analyse mntd between level2 states and "possible". This should match with the mntd function.

colnames(antitab2)
antitab2 <- antitab2[c("possible", "no", "unknown_process", "Species")]


combi <- join(medtab2,antitab2)
rownames(combi) <- combi$Species
combi[combi != 0] <- 1
combi <- combi[,-which(colnames(combi)=="Species")]
combi <- t(as.matrix(combi))

############################################ begin function
mntd.treeset <- function(trees, subset){ # a significant p-value means that the mntd of this category to the antimicrobial response category is smaller than expected by chance (=those categories are signficantly similar)
  
  if (subset == 0){
    subtrees <- trees
  }
  
  if (subset != 0){
    subtrees <- sample(trees, size=subset)
  }
  
  subtrees <- lapply(subtrees, di2multi, tol = 1e-06)  
  class(subtrees) <- "multiPhylo"
  
  phydist <- lapply(subtrees,cophenetic)
  cdis <- list()
  
  for (i in 1:length(phydist)){
    cdis[[i]] <- comdistnt(combi,phydist[[i]],abundance.weighted = FALSE, exclude.conspecifics = FALSE)
  }
  
  minus <- seq(nrow(combi)-2,2)
  t <- 20
  loc <- 20 # Change number of rows accordingly
  for (i in 1:length(minus)){
    loc <- c(loc,t+(loc[i]))
    t <- minus[i+1]
  }
  
  distances <- list()
  for (j in 1:length(cdis)){
    distances[[j]] <- "Start"
    for (i in 1:length(loc)){
      distances[[j]] <- c(distances[[j]],cdis[[j]][[loc[i]]])
    }
  }
  
  for (i in 1:length(distances)){
    distances[[i]] <- distances[[i]][-(distances[[i]] == "Start")]
    names(distances[[i]]) <- rownames(combi)[1:length(distances[[i]])]
  }
  
  # p-value 
  number_of_permutations = 999
  random_combi = combi
  resultTot <- list()
  
  for (j in 1:length(distances)){
    
    r_distances <- as.list(rep(0,length(distances[[j]])))
    names(r_distances) <- names(distances)[1:length(r_distances)]
    
    fix <- which(colSums(combi)!=0) # species, which have medicinal or antimicrobial properties
    
    for (i in 1 : number_of_permutations) {
      
      # Sample from the combined dataset without replacement
      shuffled = sample (colnames(combi), length(fix)) # sampling without replacement for only medicinal species
      
      colnames(random_combi)[fix] <- shuffled
      random_cdis <- comdistnt(random_combi, phydist[[j]], abundance.weighted = FALSE, exclude.conspecifics = FALSE)    
      
      for (k in 1:length(loc)){
        r_distances[[k]] <- c(r_distances[[k]],random_cdis[[loc[k]]])
      }
      
    }
    
    randompluscalc <- list()
    for (i in 1:length(r_distances)){
      randompluscalc[[i]] <- c(distances[[j]][i],r_distances[[i]][-1]) 
    }
    names(randompluscalc) <- rownames(combi)[1:length(randompluscalc)]
    
    pvalue <- c()
    for (i in 1:length(randompluscalc)){
      pvalue <- c(pvalue,(rank(randompluscalc[[i]])[1])/1000) # pvalue of 0.05 means that the mntd of this category was on rank 50 of 1000 compared to the randomly generated mntds, consequently the mean nearest taxon difference of this category is smaller than expected by chance, a significant pvalue for mntd means that the category is significantly related to the antimicrobial response category
    }
    
    names(pvalue) <- rownames(combi)[1:length(pvalue)]
    
    result <- cbind(distances[[j]],pvalue)  
    colnames(result) <- c("mntd", "pvalue")
    
    result <- as.data.frame(result)
    
    Signif <- symnum(as.numeric(as.character(result$pvalue)), corr = FALSE, na = FALSE, 
                     cutpoints = c(0, 0.001, 0.01, 0.05, 0.1, 1), 
                     symbols = c("***", "**", "*", ".", "ns")) 
    
    result$Sig <- Signif
    
    resultTot[[j]] <- result
  }
  
  Sum_stats <- do.call(cbind, unname(resultTot))
  
  aggr <- by(t(Sum_stats), INDICES=rownames(t(Sum_stats)), FUN=cbind)
  aggr <- as.data.frame(do.call(cbind,aggr))
  aggr.mntd <- aggr[1:20]
  indx <- sapply(aggr.mntd, is.factor)
  aggr.mntd[indx] <- lapply(aggr.mntd[indx], function(x) as.numeric(as.character(x)))
  aggr.pvalue <- aggr[21:40]
  
  median.mntd <- apply(aggr.mntd, 2, median)
  range.max <- apply(aggr.mntd, 2, max)
  range.min <- apply(aggr.mntd, 2, min)
  
  sig.pvalue <- apply(aggr.pvalue, 2, function(x) mean(x <= 0.05)) # e.g. sig.pvalue of 0.95 means that in 95% of the trees there was a p-value < 0.05, meaning that this category is significantly related to the antimicrobial response category
  
  Sums <- rbind(median.mntd,range.min,range.max,sig.pvalue)
  
  output <- list()
  output[[1]] <- resultTot
  output[[2]] <- Sums
  
  return(output)
}
############################################ end function

mntd_trees <- mntd.treeset(posteriorsample, 0) 
mntd_Complete <- as.data.frame(do.call(rbind, mntd_trees[[1]]))

write.table(mntd_Complete,"mntd_treesA.txt")
write.table(mntd_trees[[2]],"mntd_treesB.txt") 

write.csv(mntd_Complete,"mntd_treesA.csv")
write.csv(mntd_trees[[2]],"mntd_treesB.csv")
write.csv(t(mntd_trees[[2]]),"mntd_treesB_transpose.csv")

save.image("MNTD_data.RData")


####################################
#   phylogenetic diversity         #
####################################

############################################ begin function
pd.treeset <- function(trees, subset){ #
  
  if (subset == 0){
    subtrees <- trees
  }
  
  if (subset != 0){
    subtrees <- sample(trees, size=subset)
  }
  
  totalcomb <- rbind(combi,rep(1,ncol(combi)))
  rownames(totalcomb)[nrow(totalcomb)] <- "totaltree"
  PD_tot <- list()
  
  for (i in 1:length(subtrees)){
    PD <- pd(totalcomb,subtrees[[i]])
    PD$Percent <- ((PD$PD)/(PD$PD[nrow(PD)]))*100
    PD_tot[[i]] <- PD
  }
  
  Sum_PD <- do.call(cbind, unname(PD_tot))
  aggr <- by(t(Sum_PD), INDICES=rownames(t(Sum_PD)), FUN=cbind)
  aggr <- as.data.frame(do.call(cbind,aggr))
  aggr.pd <- aggr[1:nrow(totalcomb)]
  aggr.percent <- aggr[(nrow(totalcomb)+1):(2*nrow(totalcomb))]
  
  median.pd <- apply(aggr.pd, 2, median)
  range.pd.max <- apply(aggr.pd, 2, max)
  range.pd.min <- apply(aggr.pd, 2, min)
  
  median.percent <- apply(aggr.percent, 2, median)
  range.percent.max <- apply(aggr.percent, 2, max)
  range.percent.min <- apply(aggr.percent, 2, min)
  
  pd_sum <- rbind(median.pd,range.pd.min,range.pd.max)
  percent_sum <- rbind(median.percent,range.percent.min,range.percent.max)
  
  output <- list()
  output[[1]] <- PD_tot
  output[[2]] <- pd_sum
  output[[3]] <- percent_sum
  
  return(output)
}
############################################ end function

pd_trees <- pd.treeset(posteriorsample,0)
pd_Complete <- as.data.frame(do.call(rbind, pd_trees[[1]])) 

write.table(pd_Complete,"pd_treesA.txt")
write.table(pd_trees[[2]],"pd_treesB.txt")
write.table(pd_trees[[3]],"pd_treesC.txt")

write.csv(pd_Complete,"pd_treesA.csv")
write.csv(t(pd_trees[[2]]),"pd_treesB.csv")
write.csv(t(pd_trees[[3]]),"pd_treesC.csv")

save.image("phylogenetic_diversity.RData")

####################################
#   Hot nodes                      #
####################################

tree <- read.nexus("CIPRES folder/infile.nex.con.tre") # phylcom doesn't deal well with non-numeric characters in branch lengths, e.g. if 1.2e-5 change to 0.000012

## root tree
tree <-root(tree, "Arundina_graminifolia", node = NULL, resolve.root = T)

tree <- ladderize(tree, right = TRUE) # orders nodes
tree$node.label<-paste(rep("n",tree$Nnode),(length(tree$tip)+1):((length(tree$tip)*2)-1),sep="")

write.tree(tree,"ConsensusTree.txt", digits=10)

## load database
Coelogyninae_df <- read.csv("Coelogyninae_DB_RK.csv", sep=",", dec=".", header = TRUE)

## convert IDs to characters
Coelogyninae_df$tiplabels<- as.character(Coelogyninae_df$tiplabels)
Coelogyninae_df$tiplabels <- gsub("\\s","", Coelogyninae_df$tiplabels)
Coelogyninae_df$Use_Level2state<- as.character(Coelogyninae_df$Use_Level2state)
Coelogyninae_df$Use_Level2state <- gsub("\\s","", Coelogyninae_df$Use_Level2state)
Coelogyninae_df$antimicrobial.response<- as.character(Coelogyninae_df$antimicrobial.response)

## antimicrobial process
anti <- Coelogyninae_df[which(Coelogyninae_df$antimicrobial.response != "\\"),c(4,52)]
antitab <- table(anti)

## replace names in database with treetip names
antitab <- as.matrix(as.data.frame.matrix(antitab)) # species names in rows

m <- matrix(0,length(tree$tip.label[-which(tree$tip.label %in% rownames(antitab))]),ncol(antitab))
rownames(m) <- tree$tip.label[-which(tree$tip.label %in% rownames(antitab))]
phylo_anti <- rbind(antitab,m)

phylo_anti[phylo_anti != 0] <- 1 # replace all non-zero values with 1 (binary data)

phylo_anti[which(phylo_anti[,2]==1),c(1,3)] <- 0 
phylo_anti[which(phylo_anti[,1]==1),3] <- 0 

traits_anti <- phylo_anti[tree$tip.label,]
traits_anti_sample <- t(traits_anti)

writesample(traits_anti_sample,"anti_sampledata.txt") # species in columns

system("./phylocom nodesigl -f ConsensusTree.txt -s anti_sampledata.txt > anti_nodesigdata.txt")

sig <- read.table("anti_nodesigdata.txt",blank.lines.skip=F,col.names=c("plot","node","node_name","ntaxa","median","rank","sig","sig2"))
sig <- sig[-1,]

################# change according to category
sig_red <- sig[which(sig[,1]==2)[1]:which(sig[,1]==2)[length(which(sig[,1]==2))],] # 2 for possible
################ change according to category

# merge the species with the traits, in the order that they appear in the phylogeny
plotCols <- sig_red
plotCols$sig2 <- as.character(plotCols$sig2)
plotCols$node_name <- as.character(plotCols$node_name)
plotCols$plot <- as.character(plotCols$plot)
plotCols$node_number <- gsub("n","", plotCols$node_name)
plotCols$node_number <- as.numeric(plotCols$node_number)
plotCols$sig2[which(plotCols$sig2=="+")] <- 1
plotCols$sig2[which(plotCols$sig2=="-")] <- 2
plotCols$sig2[which(plotCols$sig2=="")] <- 3

plotCols$plot[which(plotCols$sig2=="1")] <- "Sig"
plotCols$plot[which(plotCols$sig2=="2")] <- "NoSig"

w <- which(plotCols$plot=="Sig")
cladesig <- list()
for (i in 1:length(w)){
  cladesig[[i]] <- tips(tree, plotCols$node_number[w[i]])
}

### only if there is NoSig
wN <- which(plotCols$plot=="NoSig")
cladesigN <- list()
for (i in 1:length(wN)){
  cladesigN[[i]] <- tips(tree, plotCols$node_number[wN[i]])
}
### only if there is NoSig

branchlist <- list()
for (i in 1:length(cladesig)){
  branchlist[[i]] <- which.edge(tree, cladesig[[i]])
}

### only if there is NoSig
branchlistN <- list()
for (i in 1:length(cladesigN)){
  branchlistN[[i]] <- which.edge(tree, cladesigN[[i]])
}
### only if there is NoSig

cols <- matrix("gray60",nrow(tree$edge),2) #gray60
for (i in 1:length(branchlist)){
  cols[branchlist[[i]],] <- "red"
}

### only if there is NoSig
for (i in 1:length(branchlistN)){
  cols[branchlistN[[i]],] <- "turquoise4"
}
### only if there is NoSig

cols <- as.vector(cols)

##################Plot tip labels with radial spacing (source: https://github.com/willpearse/willeerd/blob/master/R/phylo.plots.R; https://willeerd.wordpress.com/page/2/)
willeerd.tiplabels <- function (text, tip, adj = c(0.5, 0.5), radial.adj=1, frame = "rect", pch = NULL, thermo = NULL, pie = NULL, piecol = NULL, col = "black", bg = "yellow", horiz = FALSE, width = NULL, height = NULL, ...)
{
  lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
  if (missing(tip))
    tip <- 1:lastPP$Ntip
  lastPP$xx <- lastPP$xx * radial.adj
  lastPP$yy <- lastPP$yy * radial.adj
  XX <- lastPP$xx[tip]
  YY <- lastPP$yy[tip]
  BOTHlabels(text, tip, XX, YY, adj, frame, pch, thermo, pie,
             piecol, col, bg, horiz, width, height, ...)
}
##################Plot tip labels with radial spacing

save.image("hot_nodes.RData")

tree <- chronopl(tree, lambda = 0, age.min = 1)

#Plotting Figures

# Figure 2

#Load function
as.numeric.factor <- function(x) {as.numeric(levels(x))[x]}
#Create data.frame
lev2a<-phyloD_lev2[[1]]
lev2a<-as.data.frame(lev2a)
colnames(lev2a)<-c("Use_Level2state", "0's" , "1's" ,  "N.perm" , "D"  ,"Prob.random" , "Prob.Brownian" , "star1" , "star2" , "star3", "star4", "star_Brown")
lev2a$D<-as.numeric.factor(lev2a$D)

lev2a$Use_Level2state<-gsub("_", " ", lev2a$Use_Level2state)
lev2a$Use_Level2state<-as.factor(lev2a$Use_Level2state)
table(lev2a$D)

#Reverse levels for plotting
lev2a$Use_Level2state<-factor(lev2a$Use_Level2state, levels = rev(levels(lev2a$Use_Level2state)))
levels(lev2a$Use_Level2state)

df1<-lev2a[1:10000,]
df2<-lev2a[10001:19000,]

pdf("Figure 2 BoxPlots_19 categories of the organ targeted EBDCS_theme_hc.pdf",width=20,height=10) 
ggplot(lev2a,aes(x=Use_Level2state, y=D, fill=Use_Level2state)) +
  geom_boxplot(width=0.9) +
  geom_hline(aes(yintercept = 0, group = D), colour = 'red', linetype="dashed", size=0.8) +
  geom_hline(aes(yintercept = 1, group = D), colour = 'red', linetype="dashed", size=0.8)+theme_hc()+ scale_y_continuous(name = y, breaks = c(-5:11))+scale_fill_viridis(discrete=T) + labs(x = "Category in classification method", y= "D-statistic")+
  scale_y_continuous(name = "D-statistic")+ theme(text = element_text(size = 20)) +theme(legend.position = "None")+ coord_flip()+ theme(axis.text.x=element_text(color = "black", size=12, face="bold"))+ theme(axis.text.y=element_text(color = "black", size=12, face="bold")) +scale_y_continuous(breaks =c(-3, 0, 1, 3, 5, 7,9), limit = c(-2, 9)) #+scale_x_discrete(position = "right")
dev.off()

#Figure 3

anti2a<-phyloD_anti[[1]]
anti2a<-as.data.frame(anti2a)
colnames(anti2a)<-c("Binary_variable", "0's" , "1's" ,  "N.perm" , "D"  ,"Prob.random" , "Prob.Brownian" , "star1" , "star2" , "star3", "star4", "star_Brown")
anti2a$D<-as.numeric.factor(anti2a$D)
table(anti2a$D)
names(anti2a)
colnames(anti2a)[1]<-"Binary_variable"
levels(anti2a$Binary_variable)<-c("No response", "Possible response", "Unknown response")

df1a<-anti2a[1:10000,]
df2a<-anti2a[10001:19000,]

pdf("Figure 3 Boxplot of the Antimicrobial biological response classification method.pdf",width=10,height=8) 
ggplot(anti2a,aes(x=Binary_variable, y=D, fill=Binary_variable)) +
  geom_boxplot(width=0.9) +
  geom_hline(aes(yintercept = 0, group = D), colour = 'red', linetype="dashed", size=0.8) +
  geom_hline(aes(yintercept = 1, group = D), colour = 'red', linetype="dashed", size=0.8)+theme_hc()+ scale_y_continuous(name = y, breaks = c(-5:11)) + labs(x = "Category in classification method", y= "D-statistic")+
  scale_y_continuous(name = "D-statistic")+ theme(text = element_text(size = 20)) +theme(legend.position = "None")+ coord_flip()+ theme(axis.text.x=element_text(color = "black", size=15, face="bold"))+ theme(axis.text.y=element_text(color = "black", size=15, face="bold")) +scale_y_continuous(breaks =c(-0.4, 0, 1), limit = c(-0.3, 1.2))+scale_x_discrete(position = "right")+
  scale_fill_manual(values = c("#0000FF", "#80FF00", "#FF7F00"), guide = FALSE )
dev.off()

#Figure 4.  Median of the Phylogenetic Diversity (PD) indices (in %) 

a<-data.frame(t(pd_trees[[3]]))
a$names<-rownames(a)
a$names<-gsub("Percent.","", a$names)
a$names<-gsub("_"," ", a$names)
a<-a[-23,]
a$names[c(22,21,20,19)]<-c("Unknown antimicrobial response","No antimicrobial response","Possible antimicrobial response", "unknown EBDCS")
rownames(a)<-a$names
a<-a[c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,21,20,22),]

#Reverse levels for plotting
a$names<-as.factor(a$names) #Change into factor to order plot names
a$names<-ordered(a$names, levels=a$names) #order levels as the same as names
rownames(a)<-a$names
names(a)

a$median.percent<-round(a$median.percent, 2)

pdf("Figure 4.pdf",width=15,height=10) 
ggplot(a, aes(x=names, y=median.percent, fill=names)) + 
  geom_bar(stat="identity", position=position_dodge()) +
  geom_errorbar(aes(ymin=range.percent.min, ymax=range.percent.max), width=.2,
                position=position_dodge(.9))+ coord_flip() +
  scale_fill_manual(values = c("grey","grey","grey","grey","grey","grey","grey","grey","grey","grey","grey","grey","grey","grey","grey","grey","grey","grey","grey","#0000FF", "#80FF00", "#FF7F00"), guide = FALSE )+theme_hc()+theme(axis.title.x=element_blank(), axis.title.y=element_blank())+theme(axis.text.x=element_text(color = "black", size=12, face="bold"))+ theme(axis.text.y=element_text(color = "black", size=12, face="bold"))+ geom_text(aes(label=median.percent), vjust=-0.3, size=3.5)
dev.off()

#### Figures 1 A, B

pdf("Figure_1_B_Antimicrobial_biological_response.pdf",width=15,height=20, bg="transparent") 
pT <- plot(tree,
           show.tip.label = T, 
           show.node.label = F,
           type="fan",
           cex = 1.3,
           edge.width=2.5,
           no.margin = T,
           edge.color = "black",
           #edge.color = cols,
           font=3,
           label.offset = 0.08)
title(main="Figure_1_B-Antimicrobial_biological_response", font.main=1.5, outer=F)
nodelabels(node= plotCols$node_number[w], pch = 19, cex = 1.5, col="red")
nodelabels(node= plotCols$node_number[wN], pch = 19, cex = 1.5, col="turquoise4")
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(phylo_anti)[phylo_anti[,which(colnames(phylo_anti)=="possible")] > 0]), pch = 19, cex = 2, col = "#80FF00", radial.adj=1.02) 
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(phylo_anti)[phylo_anti[,which(colnames(phylo_anti)=="no")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02) 
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(phylo_anti)[phylo_anti[,which(colnames(phylo_anti)=="unknown_process")] > 0]), pch = 19, cex = 2, col = "#FF7F00", radial.adj=1.02) 

dev.off()

## Use_Level2state process
lev_2 <- Coelogyninae_df[which(Coelogyninae_df$Use_Level2state != "\\"),c(4,32)]
lev_2tab <- table(lev_2)

## replace namaes in database with treetip names
lev_2tab <- as.matrix(as.data.frame.matrix(lev_2tab)) # species names in rows

m <- matrix(0,length(tree$tip.label[-which(tree$tip.label %in% rownames(lev_2tab))]),ncol(lev_2tab))
rownames(m) <- tree$tip.label[-which(tree$tip.label %in% rownames(lev_2tab))]
phylo_lev_2 <- rbind(lev_2tab,m)

phylo_lev_2[phylo_lev_2 != 0] <- 1 # replace all non-zero values with 1 (binary data)

phylo_lev_2[which(phylo_lev_2[,2]==1),c(1,3)] <- 0 
phylo_lev_2[which(phylo_lev_2[,1]==1),3] <- 0 

traits_lev_2 <- phylo_lev_2[tree$tip.label,]
traits_lev_2_sample <- t(traits_lev_2)

writesample(traits_lev_2_sample,"lev_2_sampledata.txt") # species in columns

system("./phylocom nodesigl -f ConsensusTree.txt -s lev_2_sampledata.txt > lev_2_nodesigdata.txt")

sig <- read.table("lev_2_nodesigdata.txt",blank.lines.skip=F,col.names=c("plot","node","node_name","ntaxa","median","rank","sig","sig2"))
sig <- sig[-1,]

rownames(traits_lev_2_sample) # Chose category [6] "Infections_Infestations"

################# change according to category
sig_red <- sig[which(sig[,1]==6)[1]:which(sig[,1]==6)[length(which(sig[,1]==6))],] # 2 for possible
################ change according to category

# merge the species with the traits, in the order that they appear in the phylogeny
plotCols <- sig_red
plotCols$sig2 <- as.character(plotCols$sig2)
plotCols$node_name <- as.character(plotCols$node_name)
plotCols$plot <- as.character(plotCols$plot)
plotCols$node_number <- gsub("n","", plotCols$node_name)
plotCols$node_number <- as.numeric(plotCols$node_number)
plotCols$sig2[which(plotCols$sig2=="+")] <- 1
plotCols$sig2[which(plotCols$sig2=="-")] <- 2
plotCols$sig2[which(plotCols$sig2=="")] <- 3

plotCols$plot[which(plotCols$sig2=="1")] <- "Sig"
plotCols$plot[which(plotCols$sig2=="2")] <- "NoSig"

w <- which(plotCols$plot=="Sig")
cladesig <- list()
for (i in 1:length(w)){
  cladesig[[i]] <- tips(tree, plotCols$node_number[w[i]])
}

### only if there is NoSig
wN <- which(plotCols$plot=="NoSig")
cladesigN <- list()
for (i in 1:length(wN)){
  cladesigN[[i]] <- tips(tree, plotCols$node_number[wN[i]])
}
### only if there is NoSig

branchlist <- list()
for (i in 1:length(cladesig)){
  branchlist[[i]] <- which.edge(tree, cladesig[[i]])
}

### only if there is NoSig
branchlistN <- list()
for (i in 1:length(cladesigN)){
  branchlistN[[i]] <- which.edge(tree, cladesigN[[i]])
}
### only if there is NoSig

cols <- matrix("gray60",nrow(tree$edge),2) #gray60
for (i in 1:length(branchlist)){
  cols[branchlist[[i]],] <- "red"
}

### only if there is NoSig
for (i in 1:length(branchlistN)){
  cols[branchlistN[[i]],] <- "turquoise4"
}
### only if there is NoSig

cols <- as.vector(cols)

pdf("Figure_1_A-Infections_Infestation category_EBDCS.pdf",width=15,height=20, bg="transparent") 
pT <- plot(tree,
           show.tip.label = T, 
           show.node.label = F,
           type="fan",
           cex = 1.3,
           edge.width=2.5,
           no.margin = T,
           edge.color = "black",
           #edge.color = cols,
           font=3,
           label.offset = 0.08)
title(main="Figure_1_A-Infections/Infestation category-EBDCS", font.main=1.5, outer=F)
nodelabels(node= plotCols$node_number[w], pch = 19, cex = 1.5, col="red")
nodelabels(node= plotCols$node_number[wN], pch = 19, cex = 1.5, col="turquoise4")
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Circulatory_System_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Digestive_System_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Genitourinary_System_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="III_Defined_Symptoms")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Mental_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Metabolic_System_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Muscular_Skeletal_System_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Nervous_System_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Nutritional_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Pain")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Poisonings")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Respiratory_System_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Sensory_System_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Skin_Subcutaneous_Cellular_Tissue_Disorders")] > 0]), pch = 19, cex = 2, col = "#0000FF", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="Infections_Infestations")] > 0]), pch = 19, cex = 2, col = "#80FF00", radial.adj=1.02)
willeerd.tiplabels(tip = which(tree$tip.label %in% rownames(medtab)[medtab[,which(colnames(medtab)=="unknown")] > 0]), pch = 19, cex = 2, col = "#FF7F00", radial.adj=1.02)
dev.off()


save.image("Final_results_Coelogyninae.RData")

